import django
from django.utils import timezone, translation
from django.conf import settings
import pytz


class UserLanguageMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        user_language = getattr(
            request.user,
            'language',
            settings.LANGUAGE_CODE
        )
        translation.activate(user_language)

        if django.VERSION[0] <= 2:
            request.session[translation.LANGUAGE_SESSION_KEY] = user_language

        response = self.get_response(request)

        if django.VERSION[0] >= 3:
            response.set_cookie(settings.LANGUAGE_COOKIE_NAME, user_language)

        return response


class UserTimeZoneMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            timezone.activate(pytz.timezone(request.user.timezone))
        else:
            timezone.deactivate()

        return self.get_response(request)



"""
TimeZone and Language Mixin Classes
"""
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

import pytz

__all__ = (
    "UserTimeZoneSupportMixin",
    "UserLanguageSupportMixin",
    "TimeZoneAndUserLanguageSupportMixin",
)


class UserTimeZoneSupportMixin(models.Model):
    """ TimeZone Support """
    timezone = models.CharField(
        db_index=True,
        max_length=100,
        choices=[(t, t) for t in pytz.common_timezones],
        default="UTC",
        null=False,
        blank=False,
        verbose_name=_("TimeZone")
    )

    class Meta:
        abstract = True


class UserLanguageSupportMixin(models.Model):
    language = models.CharField(
        max_length=10,
        choices=settings.LANGUAGES,
        default=settings.LANGUAGE_CODE,
        verbose_name=_("Language")
    )

    class Meta:
        abstract = True


class TimeZoneAndUserLanguageSupportMixin(UserTimeZoneSupportMixin,
                                          UserLanguageSupportMixin):
    class Meta:
        abstract = True



from django.views.generic import TemplateView
from django.utils import timezone


class UserTimeZoneTestView(TemplateView):
    template_name = 'user_g11n/user_timezone.html'

    def get_context_data(self, *args, **kwargs):
        return {"datetime": timezone.now()}


class UserLanguageTestView(TemplateView):
    template_name = 'user_g11n/user_language.html'



from django.contrib.auth import get_user_model
from django.contrib.auth import models as auth_models
from django.contrib.auth.hashers import make_password
import factory


class User(factory.django.DjangoModelFactory):
    first_name = "testuser"

    last_name = factory.Sequence(lambda n: '{0}'.format(n))

    username = factory.Sequence(lambda n: 'testuser-{0}'.format(n))

    email = factory.Sequence(lambda n: 'testuser-{0}@example.com'.format(n))

    password = make_password("testtest")

    class Meta:
        model = get_user_model()



from setuptools import setup

try:
    with open("./README.md", "r") as f:
        readme = f.read()
except Exception as e:
    readme = ""


setup(
    name='django-user-g11n',
    author="salexkidd",
    author_email="salexkidd@gmail.com",
    url="https://github.com/salexkidd/django-user-g11n",
    description='User g11n extension for Django',
    long_description_content_type="text/markdown",
    long_description=readme,
    keywords=["django", "i18n", "l10n", "g11n", "timezone", ],
    version='0.3',
    packages=['user_g11n'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
